@@ -167,9 +167,6 @@ static const int signals[] = {
 	-1,
 };
 
-static void pr_error(const char *msg) __attribute__ ((noreturn));
-static void pr_nomem(const char *msg) __attribute__ ((noreturn));
-
 /* Time scaling factors */
 static const scale_t time_scales[] = {
 	{ 's',  1 },
@@ -324,7 +321,7 @@ static void get_tm(struct tm *tm)
  *  pr_error()
  *	print error message and exit fatally
  */
-static void pr_error(const char *msg)
+static void __attribute__ ((noreturn)) pr_error(const char *msg)
 {
 	fprintf(stderr, "Fatal error: %s: errno=%d (%s)\n",
 		msg, errno, strerror(errno));
@@ -335,7 +332,7 @@ static void pr_error(const char *msg)
  *  pr_nomem()
  *	print out of memory error and exit fatally
  */
-static void pr_nomem(const char *msg)
+static void __attribute__ ((noreturn)) pr_nomem(const char *msg)
 {
 	fprintf(stderr, "Fatal error: out of memory: %s\n", msg);
 	exit(EXIT_FAILURE);
@@ -872,8 +869,10 @@ static int fnotify_event_add(const struct fanotify_event_metadata *metadata)
 		pr_error("allocating fnotify filename");
 		goto tidy;
 	}
-	if (filter_out(filename))
+	if (filter_out(filename)) {
+		free(filename);
 		goto tidy;
+	}
 
 	fs = file_stat_get(filename, metadata->pid);
 	if (metadata->mask & FAN_OPEN) {
